@@ -250,7 +250,7 @@ namespace dlib
             for (size_t i = 0; i < item.size(); ++i)
                 serialize(item[i],out);
         }
-        catch (serialization_error e)
+        catch (serialization_error& e)
         { 
             throw serialization_error(e.info + "\n   while serializing object of type array"); 
         }
@@ -275,7 +275,7 @@ namespace dlib
             for (size_t i = 0; i < size; ++i)
                 deserialize(item[i],in);
         }
-        catch (serialization_error e)
+        catch (serialization_error& e)
         { 
             item.clear();
             throw serialization_error(e.info + "\n   while deserializing object of type array"); 
