@@ -20,16 +20,18 @@
 // You should also consider telling dlib to link against libjpeg, libpng, libgif, fftw, CUDA, 
 // and a BLAS and LAPACK library.  To do this you need to uncomment the following #defines.
 #define DLIB_JPEG_SUPPORT
-#define DLIB_WEBP_SUPPORT
 #define DLIB_PNG_SUPPORT
 #define DLIB_GIF_SUPPORT
+/* #undef DLIB_WEBP_SUPPORT */
+/* #undef DLIB_JXL_SUPPORT */
 /* #undef DLIB_USE_FFTW */
 /* #undef DLIB_USE_BLAS */
 /* #undef DLIB_USE_LAPACK */
 /* #undef DLIB_USE_CUDA */
 /* #undef DLIB_USE_MKL_FFT */
+#define DLIB_USE_FFMPEG
 
 // This variable allows dlib/test_for_odr_violations.h to catch people who mistakenly use
 // headers from one version of dlib with a compiled dlib binary from a different dlib version.
-#define DLIB_CHECK_FOR_VERSION_MISMATCH DLIB_VERSION_MISMATCH_CHECK__EXPECTED_VERSION_19_24_0
+#define DLIB_CHECK_FOR_VERSION_MISMATCH DLIB_VERSION_MISMATCH_CHECK__EXPECTED_VERSION_19_24_5
 
